@@ -7,16 +7,12 @@
 @homepage: https://gitlab.com/Malfatti/SciScripts
 """
 
-print('[Analysis.Analysis] Importing dependencies...')
+print('[Analysis.Analysis] Loading dependencies...')
 import numpy as np
-import os
 import scipy.signal as ssig
 import scipy.special as ssp
 from itertools import tee
-from scipy.stats.stats import pearsonr
 from scipy import fftpack
-
-from sciscripts.IO import IO
 print('[Analysis.Analysis] Done.')
 
 
@@ -133,33 +129,43 @@ def CumulativeMA(Base, Add, ElNo):
     return(Array)
 
 
-def Entropy(Hist):
+def Entropy(Signal):
+    """
+    Entropy as imprevisibility measurement and maximum entropy.
+    """
+    Hist,_ = np.histogram(Signal,np.unique(Signal).shape[0])
     Probs = Hist/np.sum(Hist)
     H = -np.sum(Probs[Probs > 0] * np.log(Probs[Probs > 0]))
-    HMax = np.log(Hist.shape[0])
+    HMax = np.log(Hist.shape[0]+1)
     return(H, HMax)
 
 
 def EucDist(A, B):
     """
-    Function to calculate the distance between two spatial points
+    Function to calculate the distance and movement direction between spatial
+    points.
 
     Parameters
     ----------
-    A and B: list or tuple or array_like
-        Iterable containing 2 elements each such as A[0] and B[0] are the
-        x coord and A[1] and B[1] are the y coord.
+    A and B: array
+        Arrays containing x (`A`) and y (`B`) coordinates. The euclidean
+        distance will be sequentially calculated for all elements in `A` and
+        `B`.
 
     Returns
     -------
     Dist: float or ndarray
         Euclidean distance between points A and B.
+
+    Ang: float or ndarray
+        Angle representing the movement direction (from -π to +π).
     """
-    Dist = ((A[1:] - A[:-1])**2 + (B[1:] - B[:-1])**2)**0.5
-    return(Dist)
+    Dist = (np.diff(A)**2 + np.diff(B)**2)**0.5
+    Ang = np.arctan2(np.diff(B), np.diff(A))
+    return(Dist, Ang)
 
 
-def FilterSignal(Signal, Rate, Frequency, Order=4, Coeff='butter', Type='bandpass'):
+def FilterSignal(Signal, Rate, Frequency, Order=4, Coeff='butter', Type='bandpass', Verbose=False):
     """
     Filter signal according to the provided parameters.
 
@@ -185,6 +191,9 @@ def FilterSignal(Signal, Rate, Frequency, Order=4, Coeff='butter', Type='bandpas
     Type: str, optional
         Type of filtering. Can be `'lowpass'`, `'highpass'` or `'bandpass'`.
 
+    Verbose: bool, optional
+        Whether or not output should be printed.
+
     Returns
     -------
     Data: array
@@ -194,7 +203,7 @@ def FilterSignal(Signal, Rate, Frequency, Order=4, Coeff='butter', Type='bandpas
     Data = np.zeros(Signal.shape, dtype='float32')
     if len(Signal.shape) == 2:
         for C in range(Signal.shape[1]):
-            print('Filtering channel', C+1, '...')
+            if Verbose: print('Filtering channel', C+1, '...')
             Data[:,C] = FilterSignal(Signal[:,C], Rate, Frequency, Order, Coeff, Type)
 
     else:
@@ -259,7 +268,7 @@ def GenTTLsRising(Rate, PulseDur, PauseBefore, PauseAfter, SampleStart, PulseNo)
     return(TTLs)
 
 
-def GetAmpEnv(Signal):
+def GetAmpEnv(Signal, RemoveOffset=True, Verbose=False):
     """
     Get amplitude envelope of a signal.
 
@@ -277,16 +286,17 @@ def GetAmpEnv(Signal):
     if len(Signal.shape) > 1:
         Data = np.zeros(Signal.shape, 'float32')
         for C in range(Data.shape[1]):
-            print(f'Processing ch {C+1}...')
-            Data[:,C] = GetAmpEnv(Signal[:,C])
+            if Verbose: print(f'Processing ch {C+1}...')
+            Data[:,C] = GetAmpEnv(Signal[:,C], RemoveOffset, Verbose)
 
     else:
         Mean = Signal.mean(axis=0)
+        s = Signal-Mean if RemoveOffset else Signal
 
         # Orders faster when signal size is a power of two, so padding and truncating
-        Data = ssig.hilbert(Signal-Mean, fftpack.next_fast_len(len(Signal)))[:len(Signal)]
+        Data = ssig.hilbert(s, fftpack.next_fast_len(len(s)))[:len(s)]
         Data = abs(Data)
-        Data += Mean
+        if RemoveOffset: Data += Mean
 
     return(Data)
 
@@ -438,7 +448,7 @@ def GetPeaks(Signal, Std=1, FixedThreshold=None):
     return(Peaks)
 
 
-def GetPhase(Signal):
+def GetPhase(Signal, Verbose=False):
     """
     Get signal phase.
 
@@ -457,7 +467,7 @@ def GetPhase(Signal):
     if len(Signal.shape) > 1:
         Data = np.zeros(Signal.shape)
         for C in range(Data.shape[1]):
-            print(f'Processing ch {C+1}...')
+            if Verbose: print(f'Processing ch {C+1}...')
             Data[:,C] = GetPhase(Signal[:,C])
 
     else:
@@ -523,6 +533,21 @@ def IsFloat(Obj):
     except: return(False)
 
 
+def ListToArrayNaNFill(List):
+    """
+    Fill arrays in `List` with NaNs until all arrays have the same
+    size, then return a 2d array.
+    """
+    MaxLen = max([len(_) for _ in List])
+    Array = np.array([
+        s if len(s)==MaxLen else np.hstack((s,[np.nan]*(MaxLen-len(s))))
+        for s in List
+    ])
+
+    return(Array)
+
+
+
 def Morlet(Freq, t, CyclesNo=5, FreqScalingFactor=1, Offset=0):
     """
     Under dev
@@ -536,6 +561,35 @@ def Morlet(Freq, t, CyclesNo=5, FreqScalingFactor=1, Offset=0):
     return(Psi)
 
 
+def MovingAverage(a, n=2):
+    """
+    Taken from Jaime
+    @ https://stackoverflow.com/a/14314054
+    """
+    ret = np.cumsum(a, dtype=float)
+    ret[n:] = ret[n:] - ret[:-n]
+    return ret[n - 1:] / n
+
+
+def MovingAvgWindow(Signal, WindowSize):
+    MWDiff = WindowSize - (Signal.shape[0] % WindowSize)
+    PadStart = MWDiff//2
+    PadEnd = (MWDiff//2)+(MWDiff%2)
+
+    if MWDiff:
+        MW = np.concatenate((
+            np.empty(PadStart)*np.nan,
+            Signal,
+            np.empty(PadEnd)*np.nan
+        ))
+    else:
+        MW = Signal.copy()
+
+    MW = MW.reshape((MW.shape[0]//WindowSize,WindowSize))
+    MW = np.repeat(np.nanmean(MW,axis=1),WindowSize)[PadStart:-PadEnd]
+    return(MW)
+
+
 def NestedClean(Obj):
     """
     Remove empty elements from nested objects.
@@ -589,7 +643,42 @@ def NestedClean(Obj):
         return(Nest)
 
 
-def Normalize(Data, MeanSubtract=False, MaxDivide=True):
+def Normalize(Data, Range=(0,1)):
+    """
+    Normalize data to the specified range.
+
+    Parameters
+    ----------
+    Data: array
+        Data to be normalized. If 2d array, each column will be processed
+        separately.
+
+    Range: iterable, optional
+        Desired minimum and maximum values to fit `Data`.
+
+    Returns
+    -------
+    Norm: array
+        Normalized data.
+
+    """
+    Norm = Data.astype('float32')
+
+    if len(Norm.shape) == 2:
+        for Ch in range(Norm.shape[1]):
+            Norm[:,Ch] = Normalize(Norm[:,Ch], Range)
+
+    elif len(Norm.shape) == 1:
+        Norm = min(Range) + (Data-np.nanmin(Data)) * (max(Range)-min(Range)) / (np.nanmax(Data)-np.nanmin(Data))
+
+    else:
+        print('Only 1 or 2 dimensions allowed.')
+        return(None)
+
+    return(Norm)
+
+
+def Normalize_Old(Data, MeanSubtract=False, MaxDivide=True):
     """
     Normalize data by subtracting the mean and/or divide by the absolute
     maximum.
@@ -616,11 +705,11 @@ def Normalize(Data, MeanSubtract=False, MaxDivide=True):
 
     if len(Norm.shape) == 2:
         for Ch in range(Norm.shape[1]):
-            Norm[:,Ch] = Normalize(Norm[:,Ch], MeanSubtract, MaxDivide)
+            Norm[:,Ch] = Normalize_Old(Norm[:,Ch], MeanSubtract, MaxDivide)
 
     elif len(Norm.shape) == 1:
-        if MeanSubtract: Norm -= Norm.mean()
-        if MaxDivide: Norm /= abs(Norm).max()
+        if MeanSubtract: Norm -= np.nanmean(Norm)
+        if MaxDivide: Norm = Norm/np.nanmax(abs(Norm)) if np.nanmax(abs(Norm)) else Norm
 
     else:
         print('Only 1 or 2 dimensions allowed.')
@@ -761,11 +850,11 @@ def StrRange(Start='a', End='z', Step=1):
         print('Only 1-char length strings are accepted.')
         return(None)
     else:
-        Range = map(chr, range(ord(Start), ord(End), Step))
+        Range = list(map(chr, range(ord(Start), ord(End), Step)))
         return(Range)
 
 
-def SubSample(Data, Rate, RateNew, t=[]):
+def DownSample(Data, Rate, RateNew, t=[]):
     """
     Resample signal to a smaller sampling rate.
 
@@ -794,26 +883,103 @@ def SubSample(Data, Rate, RateNew, t=[]):
         The resampled auxiliar array. Returned only if `t` is provided.
 
     """
-    SubSampleF = int(Rate/RateNew)
+    DownSampleF = int(Rate/RateNew)
     if len(Data.shape) > 1:
-        DataSub = Data[np.arange(0,Data.shape[0],SubSampleF), :]
+        DataSub = Data[np.arange(0,Data.shape[0],DownSampleF), :]
     else:
-        DataSub = Data[np.arange(0,Data.shape[0],SubSampleF)]
+        DataSub = Data[np.arange(0,Data.shape[0],DownSampleF)]
 
-    if len(t): tSub = t[np.arange(0,t.shape[0],SubSampleF)]
+    if len(t): tSub = t[np.arange(0,t.shape[0],DownSampleF)]
 
     if len(t): return(DataSub, tSub)
     else: return(DataSub)
 
 
-def Upsample2d(Data, Dim1, Dim2, Up):
+def Transpose(Data):
+    """
+    Transpose nested list of same length.
+
+    Parameters
+    ----------
+    Data: iterable
+        Nested iterables of the same length.
+
+    Returns
+    -------
+    T: Transposed nested iterables.
+
+    """
+    T = list(map(list, zip(*Data)))
+    return(T)
+
+
+def Upsample(Data, Dim, Up, Verbose=False):
+    """
+    Upsample array.
+
+    Parameters
+    ----------
+    Data: 1d or 2d array
+        Array of shape (Dim.shape[0],) containing data values.
+        If 2d, shape should be (Dim.shape[0], X), and
+        upsampling will be done for every X separately.
+
+    Dim: 1d array
+        Array of shape (Data.shape[0],).
+
+    Up: int
+        How many times should the dimensions be upsampled.
+
+    Verbose: bool, optional
+        Whether or not output should be printed.
+
+
+    Returns
+    -------
+    DataUp: array
+        The upsampled array.
+
+    DimUp: 1d array
+        The upsampled array for the 1st dimension.
+
+    """
+
+    DimUp = np.interp(
+        # np.linspace(0,1,len(Dim)*Up),
+        # np.linspace(0,1,len(Dim)),
+        np.arange(0,1,1/(len(Dim)*Up)),
+        np.arange(0,1,1/len(Dim)),
+        Dim
+    )
+
+    if len(Data.shape) == 2:
+        DataUp = np.zeros((len(DimUp), Data.shape[1]), dtype='float')
+        for C in range(Data.shape[1]):
+            if Verbose: print('Upsampling channel', C+1, '...')
+            DataUp[:,C] = Upsample(Data[:,C], Dim, Up, Verbose)[0]
+
+    else:
+        DataUp = np.interp(
+            # np.linspace(0,1,len(DimUp)),
+            # np.linspace(0,1,Data.shape[0]),
+            np.arange(0,1,1/len(DimUp)),
+            np.arange(0,1,1/Data.shape[0]),
+            Data
+        )
+
+    return(DataUp, DimUp)
+
+
+def Upsample2d(Data, Dim1, Dim2, Up, Verbose=False):
     """
     Upsample a 2-dimensional array.
 
     Parameters
     ----------
-    Data: 2d array
-        Array of shape (Dim1.shape[0], Dim2.shape[0]) containing the values.
+    Data: 2d or 3d array
+        Array of shape (Dim1.shape[0], Dim2.shape[0]) containing data values.
+        If 3d, shape should be (Dim1.shape[0], Dim2.shape[0], X), and
+        upsampling will be done for every X separately.
 
     Dim1: 1d array
         Array of shape (Data.shape[0],) representing the 1st dimension.
@@ -826,6 +992,9 @@ def Upsample2d(Data, Dim1, Dim2, Up):
         applied for both dimensions. If list, the values are applied to the
         dimensions, respectively.
 
+    Verbose: bool, optional
+        Whether or not output should be printed.
+
 
     Returns
     -------
@@ -841,32 +1010,52 @@ def Upsample2d(Data, Dim1, Dim2, Up):
     Example
     -------
     To upsample a spectrogram 5x in both dimension:
-        SxxUp, FUp, TUp = SxxUpsample(Sxx, F, T, 5)
+        SxxUp, FUp, TUp = Analysis.Upsample2d(Sxx, F, T, 5)
     """
-    DimsUp = [
-        np.interp(
-            np.linspace(0,1,len(Dim)*Up),
-            np.linspace(0,1,len(Dim)),
-            Dim
-        )
-        for Dim in [Dim1, Dim2]
-    ]
+    DataUp, DimsUp = [], []
 
-    Data1Up = np.zeros((len(Dim1),len(DimsUp[1])), dtype='float32')
-    for I in range(Data.shape[0]):
-        Data1Up[I,:] = np.interp(
-            np.linspace(0,1,Data1Up.shape[1]),
-            np.linspace(0,1,Data.shape[1]),
-            Data[I,:]
-        )
+    if len(Data.shape) == 3:
+        for C in range(Data.shape[2]):
+            if Verbose: print('Upsampling channel', C+1, '...')
+            d, d1, d2 = Upsample2d(Data[:,:,C], Dim1, Dim2, Up)
 
-    DataUp = np.zeros((len(DimsUp[0]),len(DimsUp[1])), dtype='float32')
-    for F in range(Data1Up.shape[1]):
-        DataUp[:,F] = np.interp(
-            np.linspace(0,1,len(DimsUp[0])),
-            np.linspace(0,1,Data1Up.shape[0]),
-            Data1Up[:,F]
-        )
+            if not len(DataUp):
+                DimsUp = [d1, d2]
+                DataUp = np.zeros((d.shape[0], d.shape[1], Data.shape[2]), dtype='float')
+
+            DataUp[:,:,C] = d
+
+    else:
+        DimsUp = [
+            np.interp(
+                # np.linspace(0,1,len(Dim)*Up),
+                # np.linspace(0,1,len(Dim)),
+                np.arange(0,1,1/(len(Dim)*Up)),
+                np.arange(0,1,1/len(Dim)),
+                Dim
+            )
+            for Dim in [Dim1, Dim2]
+        ]
+
+        Data1Up = np.zeros((len(Dim1),len(DimsUp[1])), dtype='float32')
+        for I in range(Data.shape[0]):
+            Data1Up[I,:] = np.interp(
+                # np.linspace(0,1,Data1Up.shape[1]),
+                # np.linspace(0,1,Data.shape[1]),
+                np.arange(0,1,1/Data1Up.shape[1]),
+                np.arange(0,1,1/Data.shape[1]),
+                Data[I,:]
+            )
+
+        DataUp = np.zeros((len(DimsUp[0]),len(DimsUp[1])), dtype='float32')
+        for F in range(Data1Up.shape[1]):
+            DataUp[:,F] = np.interp(
+                # np.linspace(0,1,len(DimsUp[0])),
+                # np.linspace(0,1,Data1Up.shape[0]),
+                np.arange(0,1,1/len(DimsUp[0])),
+                np.arange(0,1,1/Data1Up.shape[0]),
+                Data1Up[:,F]
+            )
 
     return(DataUp, DimsUp[0], DimsUp[1])
 
@@ -905,244 +1094,6 @@ def WhereMultiple(Test, Data):
 
 
 ## Level 1
-def ComodulationPhaseAmp(
-            Data, Rate,
-            PhaseFreqBand, PhaseFreqBandWidth, PhaseFreqBandStep,
-            AmpFreqBand, AmpFreqBandWidth, AmpFreqBandStep,
-            FilterOrder=2, Save='', Return=True
-        ):
-    """
-    Calculate how strongly Phase of one frequency band modulates amplitude of
-    another frequency band.
-
-    Parameters
-    ----------
-    Data: array
-        Data to be processed. If 2d array, each column will be processed
-        separately.
-
-    Rate: int
-        Sampling rate at which `Data` was recorded.
-
-    PhaseFreqBand: list
-        List with 2 values defining the frequency (Hz) range for phase analysis.
-
-    PhaseFreqBandWidth: int
-        Frequency width (Hz) for each frequency step for phase analysis.
-
-    PhaseFreqBandStep: int
-        Frequency (Hz) step for phase analysis.
-
-    AmpFreqBand: list
-        List with 2 values defining the frequency (Hz) range for amplitude
-        analysis.
-
-    AmpFreqBandWidth: int
-        Frequency width (Hz) for each frequency step for amplitude analysis.
-
-    AmpFreqBandStep: int
-        Frequency (Hz) step for amplitude analysis.
-
-    FilterOrder: int, optional
-        Filter order.
-
-    Save: str, optional
-        File to save the analysis results. If empty, no files will be saved.
-
-    Return: bool, optional
-        Toggle for disabling the return of the resulting analysis. Useful for
-        when running several datasets and writing the results to files.
-
-
-    Returns
-    -------
-    Comodulogram: array
-        Array with the modulation index of amplitude of each frequency to phase
-        of each frequency.
-
-    AmpFreq: array
-        Array of amplitude frequencies.
-
-    PhaseFreq: array
-        Array of phase frequencies.
-
-    """
-    if not len(Save) and not Return:
-        print('`Save` must be a path and/or `Return` must be True,')
-        print('otherwhise you are getting nothing out of this function :)')
-        return(None)
-
-    PhaseFreq = np.arange(PhaseFreqBand[0], PhaseFreqBand[1], PhaseFreqBandStep)
-    AmpFreq = np.arange(AmpFreqBand[0], AmpFreqBand[1], AmpFreqBandStep)
-
-    if len(Data.shape) == 2:
-        Comodulogram = np.empty((len(AmpFreq), len(PhaseFreq), Data.shape[1]))
-        for C in range(Data.shape[1]):
-            print(f'=== [Ch {C+1}] ===')
-            Comodulogram[:,:,C] = ComodulationPhaseAmp(Data[:,C], Rate, PhaseFreqBand, PhaseFreqBandWidth, PhaseFreqBandStep, AmpFreqBand, AmpFreqBandWidth, AmpFreqBandStep, FilterOrder, Save=False)[0]
-
-    else:
-        Phases = np.arange(-180,161,20)
-        MeanAmp = np.empty(len(Phases))
-        Comodulogram = np.empty((len(AmpFreq), len(PhaseFreq)))
-
-        Func = [GetPhase, GetAmpEnv]
-
-        print('Getting phase and amp. env. for each frequency band...', end=' ')
-        PhaseAmp = [
-            [
-                Func[A](
-                    FilterSignal(Data, Rate, [_, _+Band], FilterOrder)
-                )
-                for _ in AF
-            ]
-            for A, (AF,Band) in enumerate([
-                [PhaseFreq, PhaseFreqBandWidth],
-                [AmpFreq, AmpFreqBandWidth]
-            ])
-        ]
-        print('Done')
-
-        for PhF,PhaseF in enumerate(PhaseFreq):
-            for AF,AmpF in enumerate(AmpFreq):
-                print(f'    [Phase {PhaseF}-{PhaseF+PhaseFreqBandWidth}] [Amp {AmpF}-{AmpF+AmpFreqBandWidth}]')
-
-                for P,Ph in enumerate(Phases):
-                    I = (np.rad2deg(PhaseAmp[0][PhF])>Ph)*(np.rad2deg(PhaseAmp[0][PhF])<(Ph+20))
-                    MeanAmp[P] = np.mean(PhaseAmp[1][AF][I])
-
-                p = MeanAmp/sum(MeanAmp)
-                MI = (np.log(len(p))+sum(p[p>0]*np.log(p[p>0])))/np.log(len(p))
-                Comodulogram[AF,PhF] = MI
-
-    if len(Save):
-        if '/' in Save: os.makedirs('/'.join(Save.split('/')[:-1]), exist_ok=True)
-        IO.Bin.Write(Comodulogram, Save+'.dat')
-        IO.Bin.Write(AmpFreq, Save+'_AmpFreq.dat')
-        IO.Bin.Write(PhaseFreq, Save+'_PhaseFreq.dat')
-
-    if Return: return(Comodulogram, AmpFreq, PhaseFreq)
-    else:
-        del(Comodulogram, AmpFreq, PhaseFreq)
-        return(None)
-
-
-def ComodulationAmpAmp(
-            Data, Rate,
-            AmpFreqBand1, AmpFreqBandWidth1, AmpFreqBandStep1,
-            AmpFreqBand2, AmpFreqBandWidth2, AmpFreqBandStep2,
-            FilterOrder=2, Save='', Return=True
-        ):
-    """
-    Calculate how strongly amplitude of one frequency band modulates amplitude
-    of another frequency band.
-
-    Parameters
-    ----------
-    Data: array
-        Data to be processed. If 2d array, each column will be processed
-        separately.
-
-    Rate: int
-        Sampling rate at which `Data` was recorded.
-
-    AmpFreqBand1: list
-        List with 2 values defining the frequency (Hz) range for the first
-        amplitude analysis.
-
-    AmpFreqBandWidth1: int
-        Frequency width (Hz) for each frequency step for the first amplitude
-        analysis.
-
-    AmpFreqBandStep1: int
-        Frequency (Hz) step for the first amplitude analysis.
-
-    AmpFreqBand2: list
-        List with 2 values defining the frequency (Hz) range for the second
-        amplitude analysis.
-
-    AmpFreqBandWidth2: int
-        Frequency width (Hz) for each frequency step for the second amplitude
-        analysis.
-
-    AmpFreqBandStep2: int
-        Frequency (Hz) step for the second amplitude analysis.
-
-    FilterOrder: int, optional
-        Filter order.
-
-    Save: str, optional
-        File to save the analysis results. If empty, no files will be saved.
-
-    Return: bool, optional
-        Toggle for disabling the return of the resulting analysis. Useful for
-        when running several datasets and writing the results to files.
-
-
-    Returns
-    -------
-    Comodulogram: array
-        Array with the modulation index of amplitude of each frequency to phase
-        of each frequency.
-
-    AmpFreq1: array
-        Array of amplitude1 frequencies.
-
-    AmpFreq2: array
-        Array of amplitude2 frequencies.
-
-    """
-    if not len(Save) and not Return:
-        print('`Save` must be a path and/or `Return` must be True,')
-        print('otherwhise you are not saving the analysis results neither')
-        print('returning it. What do you expect to get from this function? :)')
-        return(None)
-
-    AmpFreq1 = np.arange(AmpFreqBand1[0], AmpFreqBand1[1], AmpFreqBandStep1)
-    AmpFreq2 = np.arange(AmpFreqBand2[0], AmpFreqBand2[1], AmpFreqBandStep2)
-
-    if len(Data.shape) == 2:
-        Comodulogram = np.empty((len(AmpFreq1), len(AmpFreq2), Data.shape[1]))
-        for C in range(Data.shape[1]):
-            print(f'=== [Ch {C+1}] ===')
-            Comodulogram[:,:,C] = ComodulationAmpAmp(Data[:,C], Rate, AmpFreqBand1, AmpFreqBandWidth1, AmpFreqBandStep1, AmpFreqBand2, AmpFreqBandWidth2, AmpFreqBandStep2, FilterOrder, Save=False)[0]
-
-    else:
-        Comodulogram = np.empty((len(AmpFreq1), len(AmpFreq2)))
-
-        print('Getting amp. env. for each frequency band...', end=' ')
-        Amp = [
-            [
-                GetAmpEnv(
-                    FilterSignal(Data, Rate, [_, _+Band], FilterOrder)
-                )
-                for _ in AF
-            ]
-            for AF,Band in [
-                [AmpFreq1, AmpFreqBandWidth1],
-                [AmpFreq2, AmpFreqBandWidth2]
-            ]
-        ]
-        print('Done')
-
-        for AF1,AmpF1 in enumerate(AmpFreq1):
-            for AF2,AmpF2 in enumerate(AmpFreq2):
-                print(f'    [Amp {AmpF1}-{AmpF1+AmpFreqBandWidth1}] [Amp {AmpF2}-{AmpF2+AmpFreqBandWidth2}]')
-                r,p = pearsonr(Amp[0][AF1], Amp[1][AF2])
-                Comodulogram[AF1,AF2] = r
-
-    if len(Save):
-        if '/' in File: os.makedirs('/'.join(File.split('/')[:-1]), exist_ok=True)
-        IO.Bin.Write(Comodulogram, File+'.dat')
-        IO.Bin.Write(AmpFreq1, File+'_AmpFreq1.dat')
-        IO.Bin.Write(AmpFreq2, File+'_AmpFreq2.dat')
-
-    if Return: return(Comodulogram, AmpFreq1, AmpFreq2)
-    else:
-        del(Comodulogram, AmpFreq1, AmpFreq2)
-        return(None)
-
-
 def CWT(
             Signal, t, Rate, Freqs, tPsi, Wavelet=Morlet,
             WaveletArgs=dict(CyclesNo=5, FreqScalingFactor=1, Offset=0)
@@ -1214,13 +1165,12 @@ def GetDeltaPhase(SignalRef, SignalRes):
     return(DeltaPhase)
 
 
-def GetEntropyMI(Hist):
+def GetEntropyMI(Signal):
     """
-    Under dev.
+    Get modulation index based on entropy.
     """
-    H = Entropy(Hist)[0]
-    Probs = Hist/np.sum(Hist)
-    MI = (np.log(Probs.shape[0])-H)/np.log(Probs.shape[0])
+    H, HMax = Entropy(Hist)
+    MI = (HMax-H)/HMax
     return(MI)
 
 
@@ -1254,7 +1204,7 @@ def GetInstFreq(Signal, Rate):
 
         # Instantaneous frequency can be achieved by
         # InstFreq = np.angle(np.exp(1j*np.diff(SignalPhase)))/(2*np.pi/Rate)
-        # Or by unwrapping the phase
+        # Or by getting the diff of the unwrapped phase
         SignalPhaseUnwrapped = np.unwrap(SignalPhase)
         InstFreq = np.diff(SignalPhaseUnwrapped)/(2*np.pi/Rate)
 
@@ -1351,7 +1301,7 @@ def GetStrongestCh(Data):
     return(BestCh)
 
 
-def PSD(Signal, Rate, Scaling='density', Window='hann', WindowSize=None, NPerSeg=None, Overlap=None):
+def PSD(Signal, Rate, Scaling='density', Window='hann', WindowSize=None, NPerSeg=None, Overlap=None, NFFT=None, Verbose=False):
     """
     Get the power spectrum density (PSD) of a signal. Built around
     `scipy.signal.welch`, so see `scipy.signal.welch?` for more info.
@@ -1383,6 +1333,13 @@ def PSD(Signal, Rate, Scaling='density', Window='hann', WindowSize=None, NPerSeg
         Number of points to overlap between segments. If `None`,
         `WindowSize//4` will be used.
 
+    NFFT: int, optional
+        Length of the FFT used, if a zero padded FFT is desired. If
+        `None`, the FFT length is calculated by Analysis.GetNFFT().
+
+    Verbose: bool, optional
+        Whether or not output should be printed.
+
 
     Returns
     -------
@@ -1393,32 +1350,33 @@ def PSD(Signal, Rate, Scaling='density', Window='hann', WindowSize=None, NPerSeg
         Power spectral density or power spectrum of x.
 
     """
-    if not WindowSize: WindowSize = Signal.shape[0]
-    if not NPerSeg: NPerSeg = WindowSize//2
-    if not Overlap: Overlap = WindowSize//4
+    if WindowSize is None: WindowSize = Signal.shape[0]
+    if NPerSeg is None: NPerSeg = WindowSize//2
+    if Overlap is None: Overlap = WindowSize//4
 
     F, PxxSp = [], []
 
     if len(Signal.shape) == 2:
         for C in range(Signal.shape[1]):
-            print('PSD of channel', C+1, '...')
-            f, pxx = PSD(Signal[:,C], Rate, Scaling, WindowSize, NPerSeg, Overlap)
+            if Verbose: print('PSD of channel', C+1, '...')
+            f, pxx = PSD(Signal[:,C], Rate, Scaling, Window, WindowSize, NPerSeg, Overlap)
 
             if not len(F):
-                F = np.zeros((len(pxx), Signal.shape[1]), dtype='float')
-                PxxSp = F.copy()
+                PxxSp = np.zeros((len(pxx), Signal.shape[1]), dtype='float')
+                F = f.copy()
 
-            F[:,C], PxxSp[:,C] = f, pxx
+
+            PxxSp[:,C] = pxx
 
     else:
-        NFFT = GetNFFT(WindowSize)
+        if NFFT is None: NFFT = GetNFFT(WindowSize)
         F, PxxSp = ssig.welch(Signal, Rate, window=Window, nperseg=NPerSeg,
                                 noverlap=Overlap, nfft=NFFT, detrend=False)
 
     return(F, PxxSp)
 
 
-def QuantifyTTLs(Data=[], StdNo=2.5, Edge='rise', FixedThreshold=None):
+def QuantifyTTLs(Data=[], StdNo=2.5, Edge='rise', FixedThreshold=None, Verbose=False):
     """
     Detect TTLs (rising or falling edges) in a signal.
 
@@ -1437,6 +1395,9 @@ def QuantifyTTLs(Data=[], StdNo=2.5, Edge='rise', FixedThreshold=None):
         If set, only edges with peak higher than `FixedThreshold` will
         be detected. In this case, `StdNo` will be ignored.
 
+    Verbose: bool, optional
+        Whether or not output should be printed.
+
 
     Returns
     -------
@@ -1444,10 +1405,10 @@ def QuantifyTTLs(Data=[], StdNo=2.5, Edge='rise', FixedThreshold=None):
         Array with indexes of detected edges in `Data`.
 
     """
-    print('Get TTL timestamps... ', end='')
+    if Verbose: print('Get TTL timestamps... ', end='')
     if not FixedThreshold: Threshold = Data.mean() + StdNo*(Data.std())
     else: Threshold = FixedThreshold
-    print('TTL threshold:', Threshold)
+    if Verbose: print('TTL threshold:', Threshold)
 
     if Edge == 'rise':
         TTLs = np.where((Data[:-1] < Threshold)*(Data[1:] > Threshold))[0]
@@ -1457,7 +1418,7 @@ def QuantifyTTLs(Data=[], StdNo=2.5, Edge='rise', FixedThreshold=None):
         print('"Edge" should be "rise" of "fall".')
         return(None)
 
-    print('Done.')
+    if Verbose: print('Done.')
     return(TTLs)
 
 
@@ -1484,12 +1445,22 @@ def RemapCh(Probe, Adaptor):
 
     """
     Probes = {
-        'A16': {'Tip': [9, 8, 10, 7, 13, 4, 12, 5, 15, 2, 16, 1, 14, 3, 11, 6],
-                'Head': [8, 7, 6, 5, 4, 3, 2, 1, 9, 10, 11, 12, 13, 14, 15, 16]},
-        'CM16': {'Tip': [9, 8, 10, 7, 11, 6, 12, 5, 13, 4, 14, 3, 15, 2, 16, 1],
-                 'Head': [5, 6, 7, 8, 9, 10, 11, 12, 1, 2, 3, 4, 13, 14, 15, 16]},
-        'Ciralli': {'Tip': [12, 11, 10, 9, 8, 7, 6, 5, 13, 14, 15, 16, 1, 2, 3, 4],
-                    'Head': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]}
+        'A1x16-A16': {
+            'Tip': [9, 8, 10, 7, 13, 4, 12, 5, 15, 2, 16, 1, 14, 3, 11, 6],
+            'Head': [8, 7, 6, 5, 4, 3, 2, 1, 9, 10, 11, 12, 13, 14, 15, 16]
+        },
+        'A1x16-CM16': {
+            'Tip': [9, 8, 10, 7, 11, 6, 12, 5, 13, 4, 14, 3, 15, 2, 16, 1],
+            'Head': [5, 6, 7, 8, 9, 10, 11, 12, 1, 2, 3, 4, 13, 14, 15, 16]
+        },
+        'A4x4-CM16': {
+            'Tip': [3, 2, 4, 1, 7, 6, 8, 5, 11, 10, 12, 9, 15, 14, 16, 13],
+            'Head': [5, 6, 7, 8, 9, 10, 11, 12, 1, 2, 3, 4, 13, 14, 15, 16]
+        },
+        'Ciralli': {
+            'Tip': [12, 11, 10, 9, 8, 7, 6, 5, 13, 14, 15, 16, 1, 2, 3, 4],
+            'Head': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]
+        }
     }
 
     Adaptors = {
@@ -1571,7 +1542,7 @@ def SignalIntensity(Signal, Rate, FreqBand, Ref, NoiseRMS=None, PSDArgs={}):
     return(Intensity, IntensityPSD)
 
 
-def Slice(Data, TTLs, SliceWindow):
+def Slice(Data, TTLs, SliceWindow, Verbose=False):
     """
     Slice data around timestamps (TTLs) according to a window.
 
@@ -1588,6 +1559,9 @@ def Slice(Data, TTLs, SliceWindow):
         List with start and end (in samples) of the slicing according to each
         timestamp in `TTLs`.
 
+    Verbose: bool, optional
+        Whether or not output should be printed.
+
 
     Returns
     -------
@@ -1602,9 +1576,9 @@ def Slice(Data, TTLs, SliceWindow):
         Array = np.zeros((NoOfSamples, len(TTLs), Data.shape[1]))
 
         for C in range(Data.shape[1]):
-            print(f'Processing {C+1} of {Data.shape[1]}...')
+            if Verbose: print(f'Processing {C+1} of {Data.shape[1]}...')
             Array[:,:,C] = Slice(Data[:,C], TTLs, [SliceWindow[0], SliceWindow[1]])
-        print('Done.')
+        if Verbose: print('Done.')
 
     else:
         NoOfSamples = abs(SliceWindow[0])+abs(SliceWindow[1])
@@ -1615,7 +1589,7 @@ def Slice(Data, TTLs, SliceWindow):
             End = TTL+SliceWindow[1]
 
             if Start < 0 or End > len(Data):
-                print('TTL too close to the edge. Skipping...')
+                if Verbose: print('TTL too close to the edge. Skipping...')
                 continue
 
             Array[:,T] = Data[Start:End]
@@ -1623,7 +1597,7 @@ def Slice(Data, TTLs, SliceWindow):
     return(Array)
 
 
-def Spectrogram(Signal, Rate, Window='hann', WindowSize=None, Overlap=None, NFFT=None):
+def Spectrogram(Signal, Rate, Window='hann', WindowSize=None, Overlap=None, NFFT=None, Verbose=False):
     """
     Get the power spectrum density (PSD) of a signal. Built around
     `scipy.signal.spectrogram`, so see `scipy.signal.spectrogram?` for more info.
@@ -1652,12 +1626,18 @@ def Spectrogram(Signal, Rate, Window='hann', WindowSize=None, Overlap=None, NFFT
         Length of the FFT to be used. If `None`, it will be automatically
         calculated based on `WindowSize` (see `sciscripts.Analysis.GetNFFT?`)
 
+    Verbose: bool, optional
+        Whether or not output should be printed.
+
 
     Returns
     -------
     F: array
         Array of sample frequencies.
 
+    T: array
+        Array of sample times.
+
     PxxSp: array
         Power spectral density or power spectrum of x.
 
@@ -1670,9 +1650,9 @@ def Spectrogram(Signal, Rate, Window='hann', WindowSize=None, Overlap=None, NFFT
 
     if len(Signal.shape) == 2:
         for C in range(Signal.shape[1]):
-            print('Spectrogram of channel', C+1, '...')
+            if Verbose: print('Spectrogram of channel', C+1, '...')
             f, t, sxx = Spectrogram(
-                Signal[:,C], Rate, WindowSize, Overlap, NFFT, Window
+                Signal[:,C], Rate, Window, WindowSize, Overlap, NFFT
             )
 
             if not len(F):
