@@ -7,7 +7,7 @@
 @homepage: https://gitlab.com/Malfatti/SciScripts
 """
 
-print('[Analysis.Plot.Plot] Importing dependencies...')
+print('[Analysis.Plot.Plot] Loading dependencies...')
 import os
 import numpy as np
 from itertools import accumulate#, combinations
@@ -54,6 +54,16 @@ def ClickCallback(Event):
     return(None)
 
 
+def ColorMix(ColorA, ColorB):
+    if max(ColorA)==0: Mix = np.array(ColorB)/2
+    elif max(ColorB)==0: Mix = np.array(ColorA)/2
+    else: Mix = np.array(ColorA)+np.array(ColorB)
+
+    if max(Mix) > 1: Mix /= max(Mix)
+    if max(Mix) == 1: Mix *= 0.85
+    return(Mix)
+
+
 def FitLegendOutside(Ax, Width=0.8, Loc='center left', Frame=False):
     # Shrink axis and fit legend outside plot
     # Taken from IanS at https://stackoverflow.com/a/4701285
@@ -128,6 +138,9 @@ def GetSpaces(Data, KeyList=None):
         if not KeyList: KeyList = sorted(Data.keys(), key=lambda x: int(x))
         Spaces = [min(Data[P[0]])-max(Data[P[1]]) for P in Pairwise(KeyList)]
 
+    else:
+        raise TypeError(f'Not sure what to do with data of type {type(Data)}')
+
     Spaces = [0]+list(accumulate(Spaces))
 
     return(Spaces)
@@ -168,46 +181,10 @@ def SameScale(Axes, Axis):
     return(None)
 
 
-def Set(Ax=None, Fig=None, AxArgs={}, FigTitle=None, Params=False,
+def Set(Ax=None, Fig=None, AxArgs={}, FigTitle=None,
         HideControls=False, Tight=True):
 
 
-    if Params:
-        Params = {
-            'image.cmap'          : 'inferno',
-            # 'savefig.dpi'         : 300,
-            'savefig.format'      : 'svg',
-            'savefig.transparent' : True,
-
-            'axes.titlesize'      : 10,
-            'xtick.labelsize'     : 'small',
-            'ytick.labelsize'     : 'small',
-            'axes.spines.top'     : False,
-            'axes.spines.right'   : False,
-            'xtick.direction'     : 'out',
-            'ytick.direction'     : 'out',
-            'lines.linewidth'     : 1,
-
-            'figure.figsize'       : FigSize,
-            # 'figure.dpi'           : Screen['DPI'][0],
-            'figure.subplot.wspace': 0.2,
-            'figure.subplot.hspace': 0.2,
-
-            'svg.fonttype'        : 'none',
-            'pdf.fonttype'        : 42,
-            'pgf.rcfonts'         : False,
-            'pgf.preamble'        : '',
-            'font.family'         : 'sans-serif',
-            'font.serif'          : ['DejaVu Serif'],
-            'font.sans-serif'     : ['DejaVu Sans'],
-            'font.cursive'        : ['Zapf Chancery'],
-            'font.monospace'      : ['DejaVu Sans Mono'],
-            'font.size'           : 10,
-            'legend.fontsize'     : 8,
-        }
-
-        return(Params)
-
     if Ax:
         ## General
         for ax in ['bottom', 'left']:
@@ -266,11 +243,10 @@ def Set(Ax=None, Fig=None, AxArgs={}, FigTitle=None, Params=False,
 
 
         ## Z
-        if 'zlim' in AxArgs:
-            if 'zlabel' in AxArgs: Ax.set_zlabel(AxArgs['zlabel'])
-            if 'zticks' in AxArgs: Ax.set_zticks(AxArgs['zticks'])
-            if 'zticklabels' in AxArgs: Ax.set_zticklabels(AxArgs['zticklabels'])
-            Ax.set_zlim(min(ZLim), max(ZLim))
+        if 'zlim' in AxArgs: Ax.set_zlim(AxArgs['zlim'])
+        if 'zlabel' in AxArgs: Ax.set_zlabel(AxArgs['zlabel'])
+        if 'zticks' in AxArgs: Ax.set_zticks(AxArgs['zticks'])
+        if 'zticklabels' in AxArgs: Ax.set_zticklabels(AxArgs['zticklabels'])
 
 
     if Fig:
@@ -292,21 +268,32 @@ def SignificanceBar(X, Y, Text, Ax, TextArgs={}, LineArgs={}, TicksDir='down', L
     elif TicksDir == 'up':
         from matplotlib.markers import TICKUP as Tick
         AmpF = LineTextSpacing if LineTextSpacing is not None else 0.98
+    elif TicksDir == 'left':
+        from matplotlib.markers import TICKLEFT as Tick
+        AmpF = LineTextSpacing if LineTextSpacing is not None else 1.02
+    elif TicksDir == 'right':
+        from matplotlib.markers import TICKRIGHT as Tick
+        AmpF = LineTextSpacing if LineTextSpacing is not None else 0.98
     elif TicksDir == None:
         Tick = None
         AmpF = LineTextSpacing if LineTextSpacing is not None else 1.02
     else:
-        print('TicksDir should be "up", "down" or None.')
-        return(None)
+        raise TypeError('TicksDir should be "up", "down", "left", "right" or None.')
 
-    TextArgsLoc = {'size': 8, 'ha': 'center', 'va': 'center'}
+    TextArgsLoc = {'ha': 'center', 'va': 'center'}
     LineArgsLoc = {'color': 'k'}
     TextArgsLoc = {**TextArgsLoc, **TextArgs}
     LineArgsLoc = {**LineArgsLoc, **LineArgs}
 
-    Yy = max(Y)*AmpF
-    Ax.plot(X, Y, marker=Tick, **LineArgsLoc)
-    Ax.text(sum(X)/2, Yy, Text, **TextArgsLoc)
+    if TicksDir in ['up', 'down', None]:
+        Yy = max(Y)*AmpF
+        Ax.plot(X, Y, marker=Tick, **LineArgsLoc)
+        Ax.text(sum(X)/2, Yy, Text, **TextArgsLoc)
+    else:
+        Xx = max(X)*AmpF
+        Ax.plot(X, Y, marker=Tick, **LineArgsLoc)
+        Ax.text(Xx, sum(Y)/2, Text, **TextArgsLoc)
+
     return(None)
 
 
@@ -325,6 +312,45 @@ FigSizeA4 = [8.27, 11.69]
 FigSize = [InchToRealSize(FigSizeA4[0]-1.97), InchToRealSize((FigSizeA4[1]-1.97)*0.3)]
 
 
+def AnovaStars(Anova, TreatmentsOrder, Ax):
+    AnovaInd = Anova['PWCs']['Factor1']['p'] < 0.05
+    if True in AnovaInd:
+        tr = Anova['PWCs']['Factor1']['Factor2'][AnovaInd]
+        ps = Anova['PWCs']['Factor1']['p'][AnovaInd]
+        g1 = Anova['PWCs']['Factor1']['group1'][AnovaInd]
+        g2 = Anova['PWCs']['Factor1']['group2'][AnovaInd]
+
+        for i,p in enumerate(ps):
+            x = TreatmentsOrder.index(tr[i])*3
+            x = [x, x+1]
+            Y = max(Ax.get_ylim())
+            SignificanceBar(x, [Y*1.05]*2, '*', Ax)
+
+    AnovaInd = Anova['PWCs']['Factor2']['p'] < 0.05
+
+    if True in AnovaInd:
+        f1 = np.array([
+            _ for a in [
+                [b]*(
+                    len(Anova['PWCs']['Factor2']['group1'])
+                    //len(Anova['PWCs']['Factor2']['Factor1'])
+                )
+                for b in Anova['PWCs']['Factor2']['Factor1']
+            ] for _ in a
+        ])
+        tr = f1[AnovaInd]
+        ps = Anova['PWCs']['Factor2']['p'][AnovaInd]
+        g1 = Anova['PWCs']['Factor2']['group1'][AnovaInd]
+        g2 = Anova['PWCs']['Factor2']['group2'][AnovaInd]
+
+        for i,p in enumerate(ps):
+            x = [TreatmentsOrder.index(_)*3+1 for _ in [g1[i], g2[i]]]
+            y = max(Ax.get_ylim())
+            SignificanceBar(x, [y*1.05]*2, '*', Ax)
+
+    return(Ax)
+
+
 def ClickXY(Frame):
     X, Y = None, None
     Fig, Ax = plt.subplots()
@@ -338,8 +364,6 @@ def ClickXY(Frame):
 def Return(Function):
     if Function.lower() == 'plt':
         if 'plt' not in globals():
-            Params = Set(Params=True)
-            from matplotlib import rcParams; rcParams.update(Params)
             from matplotlib import pyplot as plt
 
             return(plt)
@@ -475,7 +499,7 @@ def CheckData(Data, Rate, SpaceAmpF=0.3, TimeWindow=0.2, SkipTime=10, ChLabels=[
             YTicks = []
             for Ch in range(Data.shape[1]):
                 Y = Data[S:int(TimeWindow*Rate)+S,Ch] - Spaces[Ch]
-                YTicks.append(Y.mean())
+                YTicks.append(np.nanmean(Y))
                 Ax.plot(t, Y, 'k')
 
             XBar = [t[t.shape[0]//2], t[t.shape[0]//2+int(TimeWindow*0.2*Rate)]]
@@ -498,18 +522,20 @@ def CheckData(Data, Rate, SpaceAmpF=0.3, TimeWindow=0.2, SkipTime=10, ChLabels=[
 
 def FigAx(Ax, SubPlotsArgs={}):
     ReturnAx = True
+    spa = dict(constrained_layout=True)
+    spa = {**spa, **SubPlotsArgs}
     if not Ax:
         ReturnAx = False
         plt = Return('Plt')
 
-        Fig, Ax = plt.subplots(**SubPlotsArgs)
+        Fig, Ax = plt.subplots(**spa)
     else:
         Fig = None
 
     return(Fig, Ax, ReturnAx)
 
 
-def GenColorMap(RGB):
+def GenColorMap(RGB, Start=0):
     """
     Generate colormap based on the provided RGB color.
 
@@ -528,19 +554,23 @@ def GenColorMap(RGB):
 
     CMap = np.ones((256, 4))
     for RGBInd,RGBVal in enumerate(RGB):
-        CMap[:, RGBInd] = np.linspace(1, RGBVal, 256)
+        CMap[:, RGBInd] = np.linspace(Start, RGBVal, 256)
     CMap = ListedColormap(CMap)
 
     return(CMap)
 
 
-def GenLegendHandle(color='r', alpha=1, label='', hatch='', ltype='line', lw=2, marker=None):
-    if ltype == 'line':
-        Line2D = Return('Line2D')
-        Handler = Line2D([0], [0], color=color, alpha=alpha, lw=lw, label=label, marker=marker)
-    elif ltype == 'square':
-        mpatches = Return('mpatches')
-        Handler = mpatches.Patch(color=color, alpha=alpha, label=label, hatch=hatch)
+def GenLegendHandle(color='r', alpha=1, label='', linestyle='-', lw=2, marker=None):
+    Line2D = Return('Line2D')
+    Handler = Line2D([0], [0], color=color, alpha=alpha, lw=lw, label=label, marker=marker, linestyle=linestyle)
+    # if ltype == 'line':
+        # Line2D = Return('Line2D')
+        # Handler = Line2D([0], [0], color=color, alpha=alpha, lw=lw, label=label, marker=marker)
+    # elif ltype == 'marker':
+        # mpatches = Return('mpatches')
+        # Handler = mpatches.Patch(color=color, alpha=alpha, label=label, hatch=hatch, linestyle=None)
+    # else:
+        # raise Exception('`ltype` should be `line` or `marker`.')
 
     return(Handler)
 
@@ -568,7 +598,7 @@ def SaveShow(ReturnAx, Fig, Ax, AxArgs, File, Ext, Save, Show):
         return(Ax)
     else:
         plt = Return('plt')
-        Set(Ax=Ax, Fig=Fig, AxArgs=AxArgs)
+        Set(Ax=Ax, AxArgs=AxArgs)
         if Save:
             if '/' in File: os.makedirs('/'.join(File.split('/')[:-1]), exist_ok=True)
             for E in Ext: Fig.savefig(File+'.'+E)
@@ -618,9 +648,11 @@ def AllCh(Data, X=[], Colors=[], Labels=[], Leg=[], SpaceAmpF=1, ScaleBar=0, lw=
     for Ch in range(ChNo):
         Y = Data[:,Ch] + Spaces[Ch]*SpaceAmpF
         if ScaleBar:
-            if Ch == 0: Ax.plot([X[0]]*2, [0, ScaleBar], 'k', lw=lw)
+            sbX = X[0]-(X[1]-X[0])*int(X.shape[0]*0.03)
+            if Ch == 0: Ax.plot([sbX]*2, [np.nanmean(Y), -ScaleBar], 'k', lw=lw)
+
         Lines[Ch] = Ax.plot(X, Y, lw=lw)
-        YTicks[Ch] = Y.mean()
+        YTicks[Ch] = np.nanmean(Y)
 
         Lines[Ch][0].set_color(Colors[Ch])
         if Leg: Lines[Ch][0].set_label(Leg[Ch]); Ax.legend(loc='best')
@@ -637,36 +669,41 @@ def AllCh(Data, X=[], Colors=[], Labels=[], Leg=[], SpaceAmpF=1, ScaleBar=0, lw=
     return(Result)
 
 
-def BoxPlots(Data, X=[], Colors=[], LinesAmpF=1, Width=None, Means=True,
+def BoxPlots(Data, X=[], EdgeColors=[], FaceColors=[], FaceAlpha=[], LinesAmpF=1, Width=None, Means=True,
              Ax=None, AxArgs={}, File='Boxplots', Ext=['svg'], Save=False, Show=True):
 
-    if not len(X): X = list(range(1,len(Data)+1))
-    if not len(Colors): Colors = ['k']*len(Data)
+    BPData = [[_ for _ in d if not np.isnan(_)] for d in Data]
+
+    if not len(X): X = list(range(len(BPData)))
+    if not len(EdgeColors): EdgeColors = ['k']*len(BPData)
+    if not len(FaceColors): FaceColors = ['k']*len(BPData)
+    if not len(FaceAlpha): FaceAlpha = [0.2]*len(BPData)
 
     Fig, Ax, ReturnAx = FigAx(Ax)
-    BoxPlot = Ax.boxplot(Data, positions=X, widths=Width, showmeans=False,
+    BoxPlot = Ax.boxplot(BPData, positions=X, widths=Width, showmeans=False,
                          patch_artist=True)
 
     for I in range(len(BoxPlot['fliers'])):
-        BoxPlot['fliers'][I].set_markeredgecolor(Colors[I])
+        BoxPlot['fliers'][I].set_markeredgecolor(EdgeColors[I])
+        BoxPlot['fliers'][I].set_marker('.')
 
     for I in range(len(BoxPlot['boxes'])):
-        BoxPlot['boxes'][I].set(edgecolor=Colors[I], facecolor=to_rgba(Colors[I], 0.2))
+        BoxPlot['boxes'][I].set(edgecolor=EdgeColors[I], facecolor=to_rgba(FaceColors[I], FaceAlpha[I]))
 
     for I in range(len(BoxPlot['medians'])):
-        BoxPlot['medians'][I].set(color=Colors[I])
+        BoxPlot['medians'][I].set(color=EdgeColors[I])
 
     for K in ['whiskers', 'caps']:
         for I in range(len(BoxPlot[K])):
-            BoxPlot[K][I].set(color=Colors[I//2])
+            BoxPlot[K][I].set(color=EdgeColors[I//2])
 
     if Means:
         # in matplotlib 2.2.4 the 'showmeans' feature somehow prevents
         # inkscape from opening the resulting figure if exported to svg
         # so means have to be manually plotted
-        Means = [np.mean(_) for _ in Data]
+        Means = [np.mean(_) for _ in BPData]
         for B,Box in enumerate(Means):
-            Ax.plot(X[B], Box, color=Colors[B], marker='^')
+            Ax.plot(X[B], Box, color=EdgeColors[B], marker='^')
 
     AxArgs = {**{'xticks': X}, **AxArgs}
 
@@ -706,13 +743,21 @@ def MeanSEM(Data, X=[], LineArgs={}, FillArgs={},
     LineArgsDef = dict(color='k', lw=3, label='Mean')
     FillArgsDef = dict(color='k', lw=0, alpha=0.3, label='SEM')
     LA = {**LineArgsDef, **LineArgs}
-    FA = {**FillArgsDef, **FillArgs}
 
-    Mean = Data.mean(axis=1)
-    SEM = Data.std(axis=1) / (Data.shape[1]**0.5)
+    if type(Data) == np.ndarray:
+        Mean = np.nanmean(Data, axis=1)
+        SEM = np.nanstd(Data, axis=1) / (Data.shape[1]**0.5)
+    else:
+        Mean = np.array([np.nanmean(_) for _ in Data])
+        SEM = np.array([np.nanstd(_)/(len(_)**0.5) for _ in Data])
 
     if not len(X): X = np.arange(len(Mean))
-    Ax.fill_between(X, Mean+SEM, Mean-SEM, **FA)
+
+    if FillArgs is not None:
+        FA = {**FillArgsDef, **FillArgs}
+        Ax.fill_between(X, Mean+SEM, Mean-SEM, **FA)
+    else:
+        Ax.errorbar(X, Mean, SEM, **LA)
     Ax.plot(X, Mean, **LA)
 
     Result = SaveShow(ReturnAx, Fig, Ax, AArg, File, Ext, Save, Show)
@@ -745,28 +790,22 @@ def Scatter3D(
     return(Result)
 
 
-def ScatterMean(Data, X=[], ColorsMarkers=[], ColorsLines=[], Alpha=0.4, XSpread=0.1, YSpread=0, LogY=False,
-                Marker='o', Line='--', CMap=None, Paired=False, PlotArgs={}, ErrorArgs={},
-                Ax=None, AxArgs={}, File='ScatterMean', Ext=['svg'], Save=False, Show=True):
+def ScatterMean(
+        Data, X=[],
+        ColorsAvg=[], ColorsMarkers=[], ColorsLines=[], Alpha=0.4,
+        XSpread=0.1, YSpread=0, LogY=False, Mean=True,
+        Markers=[], Line='--', CMap=None, Paired=False, PlotArgs={}, ErrorArgs={},
+        Ax=None, AxArgs={}, File='ScatterMean', Ext=['svg'], Save=False, Show=True
+    ):
 
     if not len(X): X = list(range(len(Data)))
 
-    NaN = [np.isnan(_) for _ in Data]
-    if True in [True in _ for _ in NaN]:
-        if Paired:
-            NaNs = []
-            for _ in NaN:
-                NaNs = _ if len(NaNs) == 0 else NaNs + _
-
-            Data = [_[~NaNs] for _ in Data]
-        else:
-            Data = [Line[~NaN[L]] for L,Line in enumerate(Data)]
-
-    Boxes = [[np.random.uniform(V-YSpread, V+YSpread, 1)[0] for V in L] for L in Data]
+    Boxes = [[np.random.uniform(V-YSpread, V+YSpread, 1)[0] if ~np.isnan(V) else V for V in L] for L in Data]
     XPoss = [np.random.uniform(Pos-XSpread, Pos+XSpread, len(Boxes[P])) for P,Pos in enumerate(X)]
-    Errors = [[0, np.std(Box)/len(Box)**0.5, 0] for Box in Boxes]
-    Colors = [ColorsMarkers, ColorsLines]
-    ErrorArgs = {**{'zorder': np.inf}, **ErrorArgs}
+    Errors = [[0, np.nanstd(Box)/len(Box)**0.5, 0] for Box in Boxes]
+    Colors = [ColorsMarkers, ColorsLines, ColorsAvg]
+    PlotArgs = {**dict(linestyle=''), **PlotArgs}
+    ErrorArgs = {**dict(zorder=np.inf, lw=3, elinewidth=1, capsize=5), **ErrorArgs}
 
     plt = Return('plt')
     pCMap = plt.get_cmap(CMap)
@@ -775,19 +814,22 @@ def ScatterMean(Data, X=[], ColorsMarkers=[], ColorsLines=[], Alpha=0.4, XSpread
         if not len(ColorItem):
             Colors[C] = ['k']*max(len(Data), len(X))
 
+    if not len(Markers):
+        Markers = ['o']*len(Boxes)
+
     Fig, Ax, ReturnAx = FigAx(Ax)
     for P,Pos in enumerate(X):
         Box, XPos, Error = Boxes[P], XPoss[P], Errors[P]
 
         if LogY:
             pMarkers = Ax.semilogy(
-                XPos, Box, color=Colors[P], markeredgecolor=Colors[0][P],
-                markerfacecolor=Colors[0][P], marker=Marker, linestyle='', alpha=Alpha
+                XPos, Box, color=Colors[1][P], markeredgecolor=Colors[0][P],
+                markerfacecolor=Colors[0][P], marker=Markers[P], alpha=Alpha, **PlotArgs
             )
         else:
             pMarkers = Ax.plot(
                 XPos, Box, color=Colors[1][P], markeredgecolor=Colors[0][P],
-                markerfacecolor=Colors[0][P], marker=Marker, linestyle='', alpha=Alpha, **PlotArgs
+                markerfacecolor=Colors[0][P], marker=Markers[P], alpha=Alpha, **PlotArgs
             )
 
         if CMap:
@@ -799,7 +841,8 @@ def ScatterMean(Data, X=[], ColorsMarkers=[], ColorsLines=[], Alpha=0.4, XSpread
                     linestyle=pMarkers[0].get_linestyle()
                 )
 
-        Ax.errorbar([Pos-XSpread, Pos, Pos+XSpread], [np.mean(Box)]*3, Error, lw=3, elinewidth=1, capsize=5, color=Colors[0][P], **ErrorArgs)
+        if Mean:
+            Ax.errorbar([Pos-XSpread, Pos, Pos+XSpread], [np.nanmean(Box)]*3, Error, color=Colors[2][P], **ErrorArgs)
 
     if Paired:
         if len(np.unique([len(_) for _ in Data])) != 1:
@@ -822,8 +865,9 @@ def ScatterMean(Data, X=[], ColorsMarkers=[], ColorsLines=[], Alpha=0.4, XSpread
         # CMapLines = [_ for _ in Ax.get_lines() if len(_.get_xdata()) == 2]
         # ApplyColorMapToCycle(CMapLines, CMap=CMap)
 
-    Ax.set_xlim([X[0]-(XSpread*2), X[-1]+(XSpread*2)])
-    Ax.set_xticks(X)
+    if not Ax:
+        Ax.set_xlim([X[0]-(XSpread*2), X[-1]+(XSpread*2)])
+        Ax.set_xticks(X)
 
     Result = SaveShow(ReturnAx, Fig, Ax, AxArgs, File, Ext, Save, Show)
     return(Result)
@@ -855,3 +899,33 @@ def Spectrogram(T, F, Sxx, Colormap='inferno', Line={},
     Result = SaveShow(ReturnAx, Fig, Ax, AxArgs, File, Ext, Save, Show)
     return(Result)
 
+
+def Triang3D(Data, X, Y, Azimuth=-70, Elevation=30, Colormap='inferno',
+             Ax=None, AxArgs={}, File='Data3D', Ext=['svg'], Save=False, Show=True):
+    Fig, Ax, ReturnAx = FigAx(Ax, {'subplot_kw': dict(projection='3d')})
+
+    Triangulation = Return('Triangulation')
+    x = np.tile(X,2)
+
+    for LineIndex in range(len(Y)-1):
+        Y0, Y1 = Y[LineIndex], Y[LineIndex+1]
+        Data0, Data1 = Data[:,LineIndex], Data[:,LineIndex+1]
+        func = float if '.' in str(Y0) or '.' in str(Y1) else int
+
+        y = [func(Y0)]*len(Data0) + [func(Y1)]*len(Data1)
+        z = np.concatenate([Data0, Data1])
+        T = Triangulation(x, y)
+
+        Ax.plot_trisurf(
+            x, y, z, triangles=T.triangles,
+            cmap=Colormap, edgecolor='none',
+            antialiased=False, shade=False
+        )
+
+    Ax.grid(False)
+    Ax.view_init(Elevation, Azimuth)
+
+    Result = SaveShow(ReturnAx, Fig, Ax, AxArgs, File, Ext, Save, Show)
+    return(Result)
+
+
