@@ -26,10 +26,10 @@ import numpy as np
 from ast import literal_eval
 from glob import glob
 
-from sciscripts.IO.OpenEphys import OpenEphys
+from sciscripts.IO.OpenEphys import OpenEphys, SettingsXML
 
 
-def Load(Folder, Processor=None, Experiment=None, Recording=None, Unit='uV', ChannelMap=[], ImpedanceFile=''):
+def Load(Folder, Processor=None, Experiment=None, Recording=None, Unit='uV', ChannelMap=[], ImpedanceFile='', Verbose=False):
     """
     Loads data recorded by Open Ephys in Binary format as numpy memmap.
 
@@ -95,8 +95,9 @@ def Load(Folder, Processor=None, Experiment=None, Recording=None, Unit='uV', Cha
 
         then the data in Data['100']['0']['0'] will remain unaltered.
     """
+    FN = Load.__name__.replace('sciscripts','')
     Files = sorted(glob(Folder+'/**/*.dat', recursive=True))
-    InfoFiles = sorted(glob(Folder+'/*/*/structure.oebin'))
+    InfoFiles = sorted(glob(Folder+'/**/structure.oebin', recursive=True))
 
     if ImpedanceFile.lower() == 'auto':
         if glob(Folder+'/impedance_measurement.xml'):
@@ -123,7 +124,7 @@ def Load(Folder, Processor=None, Experiment=None, Recording=None, Unit='uV', Cha
         if Processor:
             if Proc != Processor: continue
 
-        print('Loading recording', int(Rec)+1, '...')
+        if Verbose: print(f'[{FN}] Loading recording {int(Rec)+1}...')
         if Exp not in Data[Proc]: Data[Proc][Exp] = {}
         Data[Proc][Exp][Rec] = np.memmap(File, dtype='int16', mode='c')
 
@@ -131,10 +132,11 @@ def Load(Folder, Processor=None, Experiment=None, Recording=None, Unit='uV', Cha
         Info = literal_eval(open(InfoFiles[F]).read())
         ProcIndex = [Info['continuous'].index(_) for _ in Info['continuous']
                      if str(_['source_processor_id']) == Proc][0]
+                     # if str(_['recorded_processor_id']) == Proc][0]
 
         ChNo = Info['continuous'][ProcIndex]['num_channels']
         if Data[Proc][Exp][Rec].shape[0]%ChNo:
-            print('Rec', Rec, 'is broken')
+            print(f'[{FN}] Rec {Rec} is broken')
             del(Data[Proc][Exp][Rec])
             continue
 
@@ -153,17 +155,18 @@ def Load(Folder, Processor=None, Experiment=None, Recording=None, Unit='uV', Cha
 
             if ChannelMap: Data[Proc][Exp] = OpenEphys.ApplyChannelMap(Data[Proc][Exp], ChannelMap)
 
-    print('Done.')
+    if Verbose: print(f'[{FN}] Done.')
 
     return(Data, Rate)
 
 
-def LoadOld(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None, Processor=None, Recording=None):
+def LoadOld(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None, Processor=None, Recording=None, Verbose=False):
     """
     Loads data recorded by Open Ephys in Binary format as numpy memmap.
     Specific for loading files recorded from a time when Open Ephys binary
     files were all on a single folder.
     """
+    FN = LoadOld.__name__.replace('sciscripts','')
     Files = sorted(glob(Folder+'/*.dat'))
     RecChs = SettingsXML.GetRecChs(Folder+'/settings.xml')[0]
 
@@ -173,8 +176,7 @@ def LoadOld(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None,
         else:
             ImpedanceFile = ''
 
-    Data = {Proc: {} for Proc in RecChs.keys()}
-    Rate = {Proc: [] for Proc in RecChs.keys()}
+    Data, Rate = {}, {}
     for File in Files:
         Exp, Proc, Rec = File.split('/')[-1][10:-4].split('_')
 
@@ -187,40 +189,53 @@ def LoadOld(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None,
         if Processor:
             if Proc != Processor: continue
 
+        if Verbose: print(f'[{FN}] Loading recording {int(Rec)+1}...')
+        if Proc not in Data.keys(): Data[Proc], Rate[Proc] = {}, {}
+        if Exp not in Data[Proc]: Data[Proc][Exp] = {}
 
-        Data[Proc][Rec] = np.memmap(File, dtype='int16')
-        Rate[Proc] = SettingsXML.GetSamplingRate(Folder+'/settings.xml')
-        Rate[Proc] = [np.array(Rate[Proc])]
+        Data[Proc][Exp][Rec] = np.memmap(File, dtype='int16')
+        Rate[Proc][Exp] = SettingsXML.GetSamplingRate(Folder+'/settings.xml')
+        Rate[Proc][Exp] = [np.array(Rate[Proc][Exp])]
 
         ChNo = len(RecChs[Proc])
-        if Data[Proc][Rec].shape[0]%ChNo:
-            print('Rec', Rec, 'is broken')
-            del(Data[Proc][Rec])
+        if Data[Proc][Exp][Rec].shape[0]%ChNo:
+            print(f'[{FN}] Rec {Rec} is broken')
+            del(Data[Proc][Exp][Rec])
             continue
 
-        SamplesPerCh = Data[Proc][Rec].shape[0]//ChNo
+        SamplesPerCh = Data[Proc][Exp][Rec].shape[0]//ChNo
 
-        Data[Proc][Rec] = Data[Proc][Rec].reshape((SamplesPerCh, ChNo))
+        Data[Proc][Exp][Rec] = Data[Proc][Exp][Rec].reshape((SamplesPerCh, ChNo))
 
     for Proc in Data.keys():
-        if Unit.lower() in ['uv', 'mv']:
-            Data[Proc] = DataTouV(Data[Proc], RecChs[Proc], Unit)
-            if ImpedanceFile:
-                Data[Proc] = VoltageToCurrent(Data[Proc], ImpedanceFile)
+        for Exp in Data[Proc].keys():
+            if Unit.lower() in ['uv', 'mv']:
+                if type(RecChs[Proc]) == dict:
+                    RecChs[Proc] = list(RecChs[Proc].values())
+                    for c in range(len(RecChs[Proc])):
+                        if 'name' in RecChs[Proc][c].keys():
+                            RecChs[Proc][c]['channel_name'] = RecChs[Proc][c]['name']
+                        if 'gain' in RecChs[Proc][c].keys():
+                            RecChs[Proc][c]['bit_volts'] = float(RecChs[Proc][c]['gain'])
 
-        if ChannelMap: Data[Proc] = OpenEphys.ApplyChannelMap(Data[Proc], ChannelMap)
-        if len(np.unique(Rate[Proc])) == 1: Rate[Proc] = Rate[Proc][0]
+                Data[Proc][Exp] = OpenEphys.BitsToVolts(Data[Proc][Exp], RecChs[Proc], Unit)
+                if ImpedanceFile:
+                    Data[Proc][Exp] = VoltageToCurrent(Data[Proc][Exp], ImpedanceFile)
+
+        if ChannelMap: Data[Proc][Exp] = OpenEphys.ApplyChannelMap(Data[Proc][Exp], ChannelMap)
+        if len(np.unique(Rate[Proc][Exp])) == 1: Rate[Proc][Exp] = Rate[Proc][Exp][0]
 
     return(Data, Rate)
 
 
-def LoadXML(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None, Processor=None, Recording=None):
+def LoadXML(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None, Processor=None, Recording=None, Verbose=False):
     """
     Loads data recorded by Open Ephys in Binary format as numpy memmap.
     Specific for loading files recorded from a time when Open Ephys binary
     files had no .oebin auxiliary file, so all info had to be extracted from
     the settings.xml file.
     """
+    FN = LoadXML.__name__.replace('sciscripts','')
     Files = sorted(glob(Folder+'/**/*.dat', recursive=True))
     RecChs = SettingsXML.GetRecChs(Folder+'/settings.xml')[0]
 
@@ -234,7 +249,7 @@ def LoadXML(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None,
     Rate = {Proc: [] for Proc in RecChs.keys()}
     for File in Files:
         Exp, Rec, _, Proc = File.split('/')[-5:-1]
-        print('Loading', Rec, '...')
+        if Verbose: print(f'[{FN}] Loading {Rec}...')
         Exp = str(int(Exp[10:])-1)
         Rec = str(int(Rec[9:])-1)
         Proc = Proc.split('.')[0].split('-')[-1]
@@ -257,7 +272,7 @@ def LoadXML(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None,
         # Data[Proc][Rec] = np.fromstring(Raw, 'int16')
         ChNo = len(RecChs[Proc])
         if Data[Proc][Rec].shape[0]%ChNo:
-            print('Rec', Rec, 'is broken')
+            print(f'[{FN}] Rec {Rec} is broken')
             del(Data[Proc][Rec])
             continue
 
@@ -267,7 +282,7 @@ def LoadXML(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None,
 
     for Proc in Data.keys():
         if Unit.lower() in ['uv', 'mv']:
-            Data[Proc] = DataTouV(Data[Proc], RecChs[Proc], Unit)
+            Data[Proc] = OpenEphys.BitsToVolts(Data[Proc], RecChs[Proc], Unit)
             if ImpedanceFile:
                 Data[Proc] = VoltageToCurrent(Data[Proc], ImpedanceFile)
 
