@@ -7,7 +7,9 @@
 @homepage: https://gitlab.com/Malfatti/SciScripts
 """
 
-print('[IO.OpenEphys.OpenEphys] Importing dependencies...')
+ScriptName = 'IO.OpenEphys.OpenEphys'
+
+print(f'[{ScriptName}] Loading dependencies...')
 import numpy as np
 from ast import literal_eval
 from glob import glob
@@ -15,7 +17,7 @@ from glob import glob
 from sciscripts.Analysis import Analysis
 from sciscripts.IO import IO, Bin, Hdf5, Txt
 from sciscripts.IO.OpenEphys import OpenEphysFormat, SettingsXML
-print('[IO.OpenEphys.OpenEphys] Done.')
+print(f'[{ScriptName}] Done.')
 
 
 ## Level 0
@@ -32,8 +34,8 @@ def ApplyChannelMap(Data, ChannelMap):
     return(Data)
 
 
-def BitsToVolts(Data, ChInfo, Unit):
-    print('Converting to uV... ', end='')
+def BitsToVolts(Data, ChInfo, Unit, Verbose=False):
+    if Verbose: print(f'Converting to {Unit}... ', end='')
     Data = {R: Rec.astype('float32') for R, Rec in Data.items()}
 
     if Unit.lower() == 'uv': U = 1
@@ -72,14 +74,6 @@ def ChooseProcs(XMLFile, Procs):
     return(Procs)
 
 
-def DataTouV(Data, RecChs, Unit):
-    print('Converting to uV... ', end='')
-    Data = {R: Rec.astype('float32') for R, Rec in Data.items()}
-    Data = BitsToVoltsXML(Data, RecChs, Unit)
-
-    return(Data)
-
-
 def EventsLoad(Folder):
     Files = sorted(glob(Folder+'/*.events'))
     if len(Files) > 1: print('Multiple sessions not supported yet.'); return(None)
@@ -102,7 +96,7 @@ def GetChNoAndProcs(File):
 
 
 def GetResistance(ImpedanceFile):
-    Impedance = SettingsXML.XML2Dict(ImpedanceFile)
+    Impedance = SettingsXML.XML.Read(ImpedanceFile)
     Impedance = {C: Ch for C,Ch in Impedance['CHANNEL'].items()}
     Chs = sorted(Impedance.keys(), key=lambda x: int(x[2:]))
 
@@ -119,7 +113,7 @@ def GetResistance(ImpedanceFile):
 
 def VoltageToCurrent(Data, ImpedanceFile):
     print('Normalizing based on resistance of each channel... ', end='')
-    Impedance = SettingsXML.XML2Dict(ImpedanceFile)
+    Impedance = SettingsXML.XML.Read(ImpedanceFile)
     Impedance = {C: Ch for C,Ch in Impedance['CHANNEL'].items()}
 
     for R in Data.keys():
@@ -131,9 +125,13 @@ def VoltageToCurrent(Data, ImpedanceFile):
 
 
 ## Level 1
-def KwikLoad(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None, Processor=None, Recording=None):
+def KwikLoad(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None, Processor=None, Recording=None, Verbose=False):
+    if not Hdf5.Avail:
+        print(f'[{ScriptName}] {e}: Please install the `hdf5` library to use this module.')
+        return(None)
+
     Kwds = sorted(glob(Folder+'/*.kwd'))
-    if Unit.lower() == 'uv':
+    if Unit.lower() in ['uv','mv']:
         XMLFile = sorted(glob(Folder+'/setting*.xml'))[0]
         RecChs = SettingsXML.GetRecChs(XMLFile)[0]
 
@@ -154,7 +152,7 @@ def KwikLoad(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None
             if Proc != Processor: continue
 
         if Proc not in Data.keys(): Data[Proc], Rate[Proc] = {}, {}
-        if Unit.lower() == 'uv':
+        if Unit.lower() in ['uv','mv']:
             Data[Proc][Exp], Attrs = Hdf5.Load('/recordings', Kwd, Copy=True)
         else:
             Data[Proc][Exp], Attrs = Hdf5.Load('/recordings', Kwd)
@@ -169,8 +167,17 @@ def KwikLoad(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None
         if len(np.unique(Rate[Proc][Exp])) == 1: Rate[Proc][Exp] = Rate[Proc][Exp][0]
 
         try:
-            if Unit.lower() == 'uv':
-                Data[Proc][Exp] = DataTouV(Data[Proc][Exp], RecChs[Proc], Unit)
+            if Unit.lower() in ['uv','mv']:
+                if type(RecChs[Proc]) == dict:
+                    RecChs[Proc] = list(RecChs[Proc].values())
+                    for c in range(len(RecChs[Proc])):
+                        if 'name' in RecChs[Proc][c].keys():
+                            RecChs[Proc][c]['channel_name'] = RecChs[Proc][c]['name']
+                        if 'gain' in RecChs[Proc][c].keys():
+                            RecChs[Proc][c]['bit_volts'] = float(RecChs[Proc][c]['gain'])
+
+                Data[Proc][Exp] = BitsToVolts(Data[Proc][Exp], RecChs[Proc], Unit)
+
                 if ImpedanceFile:
                     Data[Proc][Exp] = VoltageToCurrent(Data[Proc][Exp], ImpedanceFile)
 
@@ -179,11 +186,11 @@ def KwikLoad(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Experiment=None
 
         if ChannelMap: Data[Proc][Exp] = ApplyChannelMap(Data[Proc][Exp], ChannelMap)
 
-    print('Done.')
+    if Verbose: print('Done.')
     return(Data, Rate)
 
 
-def OELoad(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Processor=None, Recording=None):
+def OELoad(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Processor=None, Recording=None, Verbose=False):
     OEs = glob(Folder+'/*continuous')
 
     if ImpedanceFile.lower() == 'auto':
@@ -193,14 +200,14 @@ def OELoad(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Processor=None, R
             ImpedanceFile = ''
 
     Chs = [_.split('/')[-1] for _ in OEs]
-    Procs = Analysis.UniqueStr([_[:3] for _ in Chs])
+    Procs = np.unique([_[:3] for _ in Chs])
 
     Data = {_: {} for _ in Procs}; Rate = {_: {} for _ in Procs}
     Chs = {Proc: [_ for _ in Chs if _[:3] == Proc] for Proc in Procs}
 
     for P, Proc in Chs.items():
         Type = Chs[P][0].split('_')[-1].split('.')[0][:-1]
-        print(Type)
+        if Verbose: print(Type)
         Chs[P] = sorted(Proc, key=lambda x: int(x.split('_'+Type)[1].split('_')[0].split('.')[0]))
 
     for Proc in Data.keys():
@@ -223,10 +230,10 @@ def OELoad(Folder, Unit='uV', ChannelMap=[], ImpedanceFile='', Processor=None, R
             Data[Proc][R] = [OEData[_.split('.')[0]]['data'][RecInd] for _ in Chs[Proc]]
             Data[Proc][R] = np.array(Data[Proc][R]).T
 
-        if Unit.lower() == 'uv':
+        if Unit.lower() in ['uv','mv']:
             ChsInfo = [OEData[_.split('.')[0]]['header']['bitVolts'] for _ in Chs[Proc]]
             ChsInfo = {str(Ch): {'gain': BitVolt} for Ch, BitVolt in enumerate(ChsInfo)}
-            Data[Proc] = DataTouV(Data[Proc], ChsInfo)
+            Data[Proc] = BitsToVolts(Data[Proc], ChsInfo, Unit)
 
             if ImpedanceFile:
                 Data[Proc] = VoltageToCurrent(Data[Proc], ImpedanceFile)
@@ -241,6 +248,10 @@ def GetRecs(Folder):
     FilesExt = [F[-3:] for F in glob(Folder+'/*.*')]
 
     if 'kwd' in FilesExt:
+        if not Hdf5.Avail:
+            print(f'[{ScriptName}] {e}: Please install the `hdf5` library to use this module.')
+            return(None)
+
         Kwds = glob(Folder+'/*.kwd')
 
         for Kwd in Kwds:
@@ -263,7 +274,7 @@ def GetRecs(Folder):
     elif 'ous' in FilesExt:
         OEs = glob(Folder+'/*continuous')
         Chs = [_.split('/')[-1] for _ in OEs]
-        Procs = Analysis.UniqueStr([_[:3] for _ in Chs])
+        Procs = np.unique([_[:3] for _ in Chs])
         Recs = {}
 
         for Proc in Procs:
