@@ -7,7 +7,7 @@
 @homepage: https://gitlab.com/Malfatti/SciScripts
 """
 
-print('[IO.SigGen] Importing dependencies...')
+print('[IO.SigGen] Loading dependencies...')
 import numpy as np
 from scipy import signal
 
@@ -197,6 +197,8 @@ def ApplySoundAmpF(SoundPulseFiltered, Rate, SoundAmpF, NoiseFrequency,
     End = -int(round(SoundPauseAfterPulseDur*Rate))
     if End == 0: End = TotalDur
 
+    # print(TotalDur, SoundAmpF, Freqs)
+
     SoundUnit = np.zeros((TotalDur, len(SoundAmpF[list(SoundAmpF)[0]]), len(Freqs)), dtype='float32')
 
     for F,Freq in enumerate(Freqs):
@@ -208,7 +210,7 @@ def ApplySoundAmpF(SoundPulseFiltered, Rate, SoundAmpF, NoiseFrequency,
 
             IntFreq[1] = [FreqRangeStrToInt(Freq) for Freq in IntFreq[1]]
 
-            RealFKey = FreqRangeStrToInt(FKey)
+            RealFKey = FreqRangeStrToInt(Freq)
             RealFKey = min(IntFreq[1], key=lambda x:abs(x-RealFKey))
             RealFKey = IntFreq[0][IntFreq[1].index(RealFKey)]
 
