@@ -24,6 +24,29 @@ def DictFlat(Var, UpKey='', KeySep='_', Flat={}):
         return(Flat)
 
 
+def DictListsToArrays(DictLists):
+    if type(DictLists) == dict:
+        DictArrays = {k: DictListsToArrays(v) for k,v in DictLists.items()}
+    elif type(DictLists) == list and len(DictLists) and type(DictLists[0])==dict:
+        DictArrays = [{k: DictListsToArrays(v) for k,v in _.items()} for _ in DictLists]
+    else:
+        DictArrays = np.array(DictLists)
+
+    return(DictArrays)
+
+def DictObjToList(Dict):
+    for K,V in Dict.items():
+        if type(V) == dict: Dict[K] = DictObjToList(V)
+
+        elif type(V) == np.ndarray and V.dtype == np.dtype('O'):
+            Dict[K] = V.tolist()
+            if K in ('p', 'p.adj') and None in V:
+                Dict[K] = np.array([_ if _ is not None else np.nan for _ in V])
+
+        else: pass
+
+    return(Dict)
+
 def Print(value, htchar='    ', itemchar=' ', breaklineat='auto', lfchar='\n', indent=0):
     ''' Modified from y.petremann's code.
         Added options to set item separator for list or tuple and to set a number
