@@ -7,7 +7,7 @@
 @homepage: https://gitlab.com/Malfatti/SciScripts
 """
 
-print('[IO.Wav] Importing dependencies...')
+print('[IO.Wav] Loading dependencies...')
 import os
 
 from scipy.io import wavfile
